# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import config_pb2 as config__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0emessages.proto\x12\x14\x61ws.kinesis.protobuf\x1a\x0c\x63onfig.proto\"!\n\x03Tag\x12\x0b\n\x03key\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x01(\t\"}\n\x06Record\x12\x1b\n\x13partition_key_index\x18\x01 \x02(\x04\x12\x1f\n\x17\x65xplicit_hash_key_index\x18\x02 \x01(\x04\x12\x0c\n\x04\x64\x61ta\x18\x03 \x02(\x0c\x12\'\n\x04tags\x18\x04 \x03(\x0b\x32\x19.aws.kinesis.protobuf.Tag\"\x7f\n\x10\x41ggregatedRecord\x12\x1b\n\x13partition_key_table\x18\x01 \x03(\t\x12\x1f\n\x17\x65xplicit_hash_key_table\x18\x02 \x03(\t\x12-\n\x07records\x18\x03 \x03(\x0b\x32\x1c.aws.kinesis.protobuf.Record\"\xe6\x03\n\x07Message\x12\n\n\x02id\x18\x01 \x02(\x04\x12\x11\n\tsource_id\x18\x02 \x01(\x04\x12\x35\n\nput_record\x18\x03 \x01(\x0b\x32\x1f.aws.kinesis.protobuf.PutRecordH\x00\x12,\n\x05\x66lush\x18\x04 \x01(\x0b\x32\x1b.aws.kinesis.protobuf.FlushH\x00\x12\x42\n\x11put_record_result\x18\x05 \x01(\x0b\x32%.aws.kinesis.protobuf.PutRecordResultH\x00\x12<\n\rconfiguration\x18\x06 \x01(\x0b\x32#.aws.kinesis.protobuf.ConfigurationH\x00\x12?\n\x0fmetrics_request\x18\x07 \x01(\x0b\x32$.aws.kinesis.protobuf.MetricsRequestH\x00\x12\x41\n\x10metrics_response\x18\x08 \x01(\x0b\x32%.aws.kinesis.protobuf.MetricsResponseH\x00\x12?\n\x0fset_credentials\x18\t \x01(\x0b\x32$.aws.kinesis.protobuf.SetCredentialsH\x00\x42\x10\n\x0e\x61\x63tual_message\"`\n\tPutRecord\x12\x13\n\x0bstream_name\x18\x01 \x02(\t\x12\x15\n\rpartition_key\x18\x02 \x02(\t\x12\x19\n\x11\x65xplicit_hash_key\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x02(\x0c\"\x1c\n\x05\x46lush\x12\x13\n\x0bstream_name\x18\x01 \x01(\t\"f\n\x07\x41ttempt\x12\r\n\x05\x64\x65lay\x18\x01 \x02(\r\x12\x10\n\x08\x64uration\x18\x02 \x02(\r\x12\x0f\n\x07success\x18\x03 \x02(\x08\x12\x12\n\nerror_code\x18\x04 \x01(\t\x12\x15\n\rerror_message\x18\x05 \x01(\t\"~\n\x0fPutRecordResult\x12/\n\x08\x61ttempts\x18\x01 \x03(\x0b\x32\x1d.aws.kinesis.protobuf.Attempt\x12\x0f\n\x07success\x18\x02 \x02(\x08\x12\x10\n\x08shard_id\x18\x03 \x01(\t\x12\x17\n\x0fsequence_number\x18\x04 \x01(\t\">\n\x0b\x43redentials\x12\x0c\n\x04\x61kid\x18\x01 \x02(\t\x12\x12\n\nsecret_key\x18\x02 \x02(\t\x12\r\n\x05token\x18\x03 \x01(\t\"]\n\x0eSetCredentials\x12\x13\n\x0b\x66or_metrics\x18\x01 \x01(\x08\x12\x36\n\x0b\x63redentials\x18\x02 \x02(\x0b\x32!.aws.kinesis.protobuf.Credentials\"\'\n\tDimension\x12\x0b\n\x03key\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\"K\n\x05Stats\x12\r\n\x05\x63ount\x18\x01 \x02(\x01\x12\x0b\n\x03sum\x18\x02 \x02(\x01\x12\x0c\n\x04mean\x18\x03 \x02(\x01\x12\x0b\n\x03min\x18\x04 \x02(\x01\x12\x0b\n\x03max\x18\x05 \x02(\x01\"\x88\x01\n\x06Metric\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x33\n\ndimensions\x18\x02 \x03(\x0b\x32\x1f.aws.kinesis.protobuf.Dimension\x12*\n\x05stats\x18\x03 \x02(\x0b\x32\x1b.aws.kinesis.protobuf.Stats\x12\x0f\n\x07seconds\x18\x04 \x02(\x04\"/\n\x0eMetricsRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07seconds\x18\x02 \x01(\x04\"@\n\x0fMetricsResponse\x12-\n\x07metrics\x18\x01 \x03(\x0b\x32\x1c.aws.kinesis.protobuf.MetricB2\n0com.amazonaws.services.kinesis.producer.protobuf')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'messages_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n0com.amazonaws.services.kinesis.producer.protobuf'
  _TAG._serialized_start=54
  _TAG._serialized_end=87
  _RECORD._serialized_start=89
  _RECORD._serialized_end=214
  _AGGREGATEDRECORD._serialized_start=216
  _AGGREGATEDRECORD._serialized_end=343
  _MESSAGE._serialized_start=346
  _MESSAGE._serialized_end=832
  _PUTRECORD._serialized_start=834
  _PUTRECORD._serialized_end=930
  _FLUSH._serialized_start=932
  _FLUSH._serialized_end=960
  _ATTEMPT._serialized_start=962
  _ATTEMPT._serialized_end=1064
  _PUTRECORDRESULT._serialized_start=1066
  _PUTRECORDRESULT._serialized_end=1192
  _CREDENTIALS._serialized_start=1194
  _CREDENTIALS._serialized_end=1256
  _SETCREDENTIALS._serialized_start=1258
  _SETCREDENTIALS._serialized_end=1351
  _DIMENSION._serialized_start=1353
  _DIMENSION._serialized_end=1392
  _STATS._serialized_start=1394
  _STATS._serialized_end=1469
  _METRIC._serialized_start=1472
  _METRIC._serialized_end=1608
  _METRICSREQUEST._serialized_start=1610
  _METRICSREQUEST._serialized_end=1657
  _METRICSRESPONSE._serialized_start=1659
  _METRICSRESPONSE._serialized_end=1723
# @@protoc_insertion_point(module_scope)
