# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: config.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x63onfig.proto\x12\x14\x61ws.kinesis.protobuf\"F\n\x13\x41\x64\x64itionalDimension\x12\x0b\n\x03key\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\t\x12\x13\n\x0bgranularity\x18\x03 \x02(\t\"\x90\t\n\rConfiguration\x12J\n\x16\x61\x64\x64itional_metric_dims\x18\x80\x01 \x03(\x0b\x32).aws.kinesis.protobuf.AdditionalDimension\x12!\n\x13\x61ggregation_enabled\x18\x01 \x01(\x08:\x04true\x12)\n\x15\x61ggregation_max_count\x18\x02 \x01(\x04:\n4294967295\x12#\n\x14\x61ggregation_max_size\x18\x03 \x01(\x04:\x05\x35\x31\x32\x30\x30\x12\x1d\n\x13\x63loudwatch_endpoint\x18\x04 \x01(\t:\x00\x12\x1c\n\x0f\x63loudwatch_port\x18\x05 \x01(\x04:\x03\x34\x34\x33\x12!\n\x14\x63ollection_max_count\x18\x06 \x01(\x04:\x03\x35\x30\x30\x12$\n\x13\x63ollection_max_size\x18\x07 \x01(\x04:\x07\x35\x32\x34\x32\x38\x38\x30\x12\x1d\n\x0f\x63onnect_timeout\x18\x08 \x01(\x04:\x04\x36\x30\x30\x30\x12 \n\x11\x65nable_core_dumps\x18\t \x01(\x08:\x05\x66\x61lse\x12 \n\x11\x66\x61il_if_throttled\x18\n \x01(\x08:\x05\x66\x61lse\x12\x1a\n\x10kinesis_endpoint\x18\x0b \x01(\t:\x00\x12\x19\n\x0ckinesis_port\x18\x0c \x01(\x04:\x03\x34\x34\x33\x12\x17\n\tlog_level\x18\r \x01(\t:\x04info\x12\x1b\n\x0fmax_connections\x18\x0e \x01(\x04:\x02\x32\x34\x12\"\n\x13metrics_granularity\x18\x0f \x01(\t:\x05shard\x12\x1f\n\rmetrics_level\x18\x10 \x01(\t:\x08\x64\x65tailed\x12\x31\n\x11metrics_namespace\x18\x11 \x01(\t:\x16KinesisProducerLibrary\x12#\n\x14metrics_upload_delay\x18\x12 \x01(\x04:\x05\x36\x30\x30\x30\x30\x12\x1a\n\x0fmin_connections\x18\x13 \x01(\x04:\x01\x31\x12\x17\n\nrate_limit\x18\x14 \x01(\x04:\x03\x31\x35\x30\x12%\n\x18record_max_buffered_time\x18\x15 \x01(\x04:\x03\x31\x30\x30\x12\x19\n\nrecord_ttl\x18\x16 \x01(\x04:\x05\x33\x30\x30\x30\x30\x12\x10\n\x06region\x18\x17 \x01(\t:\x00\x12\x1d\n\x0frequest_timeout\x18\x18 \x01(\x04:\x04\x36\x30\x30\x30\x12 \n\x12verify_certificate\x18\x19 \x01(\x08:\x04true\x12\x14\n\nproxy_host\x18\x1a \x01(\t:\x00\x12\x17\n\nproxy_port\x18\x1b \x01(\x04:\x03\x34\x34\x33\x12\x19\n\x0fproxy_user_name\x18\x1c \x01(\t:\x00\x12\x18\n\x0eproxy_password\x18\x1d \x01(\t:\x00\x12T\n\rthread_config\x18\x1e \x01(\x0e\x32\x30.aws.kinesis.protobuf.Configuration.ThreadConfig:\x0bPER_REQUEST\x12\x1c\n\x10thread_pool_size\x18\x1f \x01(\r:\x02\x36\x34\"+\n\x0cThreadConfig\x12\x0f\n\x0bPER_REQUEST\x10\x00\x12\n\n\x06POOLED\x10\x01\x42\x32\n0com.amazonaws.services.kinesis.producer.protobuf')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'config_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n0com.amazonaws.services.kinesis.producer.protobuf'
  _ADDITIONALDIMENSION._serialized_start=38
  _ADDITIONALDIMENSION._serialized_end=108
  _CONFIGURATION._serialized_start=111
  _CONFIGURATION._serialized_end=1279
  _CONFIGURATION_THREADCONFIG._serialized_start=1236
  _CONFIGURATION_THREADCONFIG._serialized_end=1279
# @@protoc_insertion_point(module_scope)
